<?php
/*
 * @package Joomla 1.5
 * @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 *
 * @component XMovie Component
 * @copyright Copyright (C) Dana Harris optikool.com
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
 */
class MovieHelper {
	/**
	 * Function to get the users access level
	 * 
	 * @return type The list of access levels
	 */
	function getAccessLevel() {
		$user =& JFactory::getUser();
		$type = '';
		switch($user->usertype) {
			case '':
				$type = " AND access='0'";
				break;
			case 'Registered':
				$type = " AND (access='0' OR access='1')";
				break;			
			default:
				$type = " AND (access='0' OR access='1' OR access='2')";
				break;
		}
		return $type;
	}
	
	/**
	 * Function to get both movie and category access levels.
	 * 
	 * @return type The list of access levels
	 */
	function getAccessLevelCC() {
		$user =& JFactory::getUser();
		$type = '';
		switch($user->usertype) {
			case '':
				$typeCat = " AND #__xmovie_categories.access='0'";
				$typeCol = " AND #__xmovie_movies.access='0'";
				break;
			case 'Registered':
				$typeCat = " AND (#__xmovie_categories.access='0' OR #__xmovie_categories.access='1')";
				$typeCol = " AND (#__xmovie_movies.access='0' OR #__xmovie_movies.access='1')";
				break;			
			default:
				$typeCat = " AND (#__xmovie_categories.access='0' OR #__xmovie_categories.access='1' OR #__xmovie_categories.access='2')";
				$typeCol = " AND (#__xmovie_movies.access='0' OR #__xmovie_movies.access='1' OR #__xmovie_movies.access='2')";
				break;
		}
		
		$type = $typeCol . $typeCat;
		return $type;
	}
	
	/**
	 * Function to check access level on categories
	 *
	 */
	function checkAccessLevel( $cat ) {
		global $option;
		$user =& JFactory::getUser();
		$type = '';
		
		$query = "SELECT access FROM #__xmovie_categories WHERE published = '1' AND id ='{$cat}' LIMIT 1";
		$row = $this->_getList($query);
		
		switch($user->usertype) {
			case '':
				$type = array('0');
				break;
			case 'Registered':
				$type = array('0', '1');
				break;			
			default:
				$type = array('0', '1', '2');
				break;
		}

		if(!in_array($row[0]->access, $type)) {
			$itemid = MovieHelper::getXMovieItemId();
			$slug = $cat . ":" . MovieHelper::getCategorySlug();					 
			$link = "index.php?option=com_xmovie&view=category&id={$slug}&Itemid={$itemid->id}";
			$redirectUrl = base64_encode($link);
			$redirectUrl = '&return='.$redirectUrl;	
			$joomlaLoginUrl = 'index.php?option=com_user&view=login';				
            $finalUrl = $joomlaLoginUrl . $redirectUrl;
					
			header('Location: '.JRoute::_($finalUrl));			
		}
	}
	
	/**
	 * Function to to check Movie access level
	 */
	function checkMovieAccess( $id ) {
		global $option;
		
		$user =& JFactory::getUser();
		$type = '';
		
		$query = "SELECT access FROM #__xmovie_movies WHERE published = '1' AND id ='{$id}' LIMIT 1";
		$row = $this->_getList($query);
		
		switch($user->usertype) {
			case '':
				$type = array('0');
				break;
			case 'Registered':
				$type = array('0', '1');
				break;			
			default:
				$type = array('0', '1', '2');
				break;
		}

		if(!in_array($row[0]->access, $type)) {
			$itemid = MovieHelper::getXMovieItemId();
			$cat = MovieHelper::getCategory($id);
			$slugcat = $cat->cid . ":" . MovieHelper::getCategorySlug();
			$slug = $id . ":" . MovieHelper::getMovieSlug();					 
			$link = "index.php?option=com_xmovie&view=single&catid={$slugcat}&id={$slug}&Itemid={$itemid->id}";
			$redirectUrl = base64_encode($link);
			$redirectUrl = '&return='.$redirectUrl;	
			$joomlaLoginUrl = 'index.php?option=com_user&view=login';				
            $finalUrl = $joomlaLoginUrl . $redirectUrl;
					
			header('Location: '.JRoute::_($finalUrl));			
		}
	}
	
	/**
	 * Function to format links for SEO
	 *
	 * @return link The formatted link
	 */
	function getRoute( $link ) {
		$link = JRoute::_($link);
		return $link;
	}
	
	/**
	 * Function to return the visitors browser type.
	 *
	 * @return browser Array with the browser version and name
	 */
	function getBrowserType() {
		$useragent = $_SERVER['HTTP_USER_AGENT'];
		$browser = array();
		if (preg_match('|MSIE ([0-9].[0-9]{1,2})|',$useragent,$matched)) {
    		$browser['browser_version'] = $matched[1];
    		$browser['browser'] = 'IE';
		} elseif (preg_match( '|Opera ([0-9].[0-9]{1,2})|',$useragent,$matched)) {
    		$browser['browser_version'] = $matched[1];
    		$browser['browser'] = 'Opera';
		} elseif(preg_match('|Firefox/([0-9\.]+)|',$useragent,$matched)) {
        	$browser['browser_version'] = $matched[1];
        	$browser['browser'] = 'Firefox';
		} elseif(preg_match('|Safari/([0-9\.]+)|',$useragent,$matched)) {
        	$browser['browser_version'] = $matched[1];
        	$browser['browser'] = 'Safari';
		} else {
        	// browser not recognized!
    		$browser['browser_version'] = 0;
    		$browser['browser'] = 'other';
		}
		
		return $browser;
	}
	
	/**
	 * Function to get the embed code for the video.
	 *
	 * @return
	 */
	function getEmbedCode(&$movie) {
		$component = JComponentHelper::getComponent( 'com_xmovie' );
  		$cfgParams = new JParameter( $component->params );
		
		if($cfgParams->get('enable_movie_background', 0)) {
  			$url = parse_url($movie->thumb);
			
  			if(!array_key_exists('host', $url)) {
				$thumbPath = JURI::root().$cfgParams->get('movie_thumb_path', 'images/stories') . DS .$movie->thumb;
  			} else {
  				$thumbPath = $movie->thumb;
  			}
  		} else {
  			$thumbPath = '';
  		}
  		
		if($cfgParams->get('cview_autoplay', 0)) {
			$autoplay = "true";
		} else {
			$autoplay = "false";
		}
		$autoplaystring = "&autoplay={$autoplay}";
		$embed = '';
		$size = MovieHelper::calDimenions($movie->width, $movie->height, $cfgParams->get('single_max_width', 1028), $cfgParams->get('single_max_height', 768));
		
		switch($movie->type) {
			case '1':	// You Tube				
				$movie->link = $movie->link.$autoplaystring;
				$embed = "<object width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='allowFullScreen' value='true'></param>";
				$embed .= "<param name='allowscriptaccess' value='always'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$flashvars = '';
				if($movie->flashvars != '') {
					$embed .= "<param name='flashVars' value='{$movie->flashvars}' />";
					$flashvars = "flashvars='{$movie->flashvars}' ";
				}
				$embed .= "<embed wmode='transparent' src='{$movie->link}' type='application/x-shockwave-flash' allowscriptaccess='always' {$flashvars} allowfullscreen='true' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}'>";
				$embed .= "</embed></object>";				
				break;
			case '2':	// Daily Motion
				$embed = "<object width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<param name='allowFullScreen' value='true'></param>";
				$embed .= "<param name='allowScriptAccess' value='always'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<embed src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' allowfullscreen='true' allowscriptaccess='always' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";			
				break;
			case '3':	// Meta Cafe
				$embed = "<object width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='allowFullScreen' value='true'></param>";
				$embed .= "<param name='allowScriptAccess' value='always'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<embed src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' allowfullscreen='true' allowscriptaccess='always' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";			
				break;
			case '4':	// My Video
				$embed = "<object style='width:{$size['width']}px;height:285px;' width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='AllowFullscreen' value='true'></param>";
				$embed .= "<param name='AllowScriptAccess' value='always'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<embed src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' type='application/x-shockwave-flash' allowscriptaccess='always' allowfullscreen='true' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";			
				break;
			case '5':	// Yahoo Video
				$embed = "<object width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}' />";
				$embed .= "<param name='allowFullScreen' value='true' />";
				$embed .= "<param name='AllowScriptAccess' VALUE='always' />";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<param name='bgcolor' value='#000000' />";
				$embed .= "<param name='flashVars' value='{$movie->flashvars}' />";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<embed src='{$movie->link}' type='application/x-shockwave-flash' width='{$size['width']}' height='{$size['height']}' allowFullScreen='true' AllowScriptAccess='always' bgcolor='#000000' flashVars='{$movie->flashvars}' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";			
				break;
			case '6':	// Veoh
				$embed = "<object width='{$size['width']}' height='{$size['height']}' id='veohFlashPlayer' name='veohFlashPlayer'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='allowFullScreen' value='true'></param>";
				$embed .= "<param name='allowscriptaccess' value='always'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<embed src='{$movie->link}' type='application/x-shockwave-flash' allowscriptaccess='always' allowfullscreen='true' width='{$size['width']}' height='{$size['height']}' id='veohFlashPlayerEmbed' name='veohFlashPlayerEmbed' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";				
				break;
			case '7':	// PP2G TV
				$embed = "<object classid='clsid:d27cdb6e-ae6d-11cf-96b8-444553540000' width='{$size['width']}' height='{$size['height']}'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<param name='flashvars' value='width={$size['width']}&height={$size['height']}'></param>";
				$embed .= "<embed src='{$movie->link}' type='application/x-shockwave-flash' wmode='transparent' width='{$size['width']}' height='{$size['height']}' flashvars='width={$size['width']}&height={$size['height']}' autoplay='{$autoplay}'></embed></object>";
				break;
			case '8':	// Watch Me TV
				$embed = "<object classid='clsid:d27cdb6e-ae6d-11cf-96b8-444553540000' id='external55fa6b030b5add10de4e5c8d6ae8b5ff' codebase='http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,115,0' width='{$size['width']}' height='{$size['height']}' align='middle'>";
				$embed .= "<param name='movie' value='{$movie->link}'></param>";
				$embed .= "<param name='allowScriptAccess' value='always' />";
				$embed .= "<param name='autoplay' value='{$autoplay}'></param>";
				$embed .= "<param name='wmode' value='transparent'></param>";
				$embed .= "<embed src='{$movie->link}' type='application/x-shockwave-flash' width='{$size['width']}' height='{$size['height']}' allowScriptAccess='always' autoplay='{$autoplay}' wmode='transparent'></embed>";
				$embed .= "</object>";			
				break;
			case '9':	// DIVX Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embed .= "<param name='custommode' value='none' />";
				$embed .= "<param name='autoPlay' value='{$autoplay}' />";
				$embed .= "<param name='src' value='{$link}' />";
				$embed .= "<embed type='video/divx' src='{$link}' custommode='none' width='{$size['width']}' height='{$size['height']}'  autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embed .= "</embed>";
				$embed .= "</object>";
				break;
			case '10':	// AVI Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;				
				/*
				$bType = MovieHelper::getBrowserType();				
				if($bType['browser'] == 'Firefox' or $bType['browser'] == 'Safari') {
					$bUrl = pathinfo($movie->link);
					$embed = "<object type='video/quicktime' data='{$link}' width='{$size['width']}' height='{$size['height']}'>";
					$embed .= "<param name='controller' value='false' >";
					$embed .= "<param name='autoplay' value='true'>";
					$embed .= "alt : <a href='{$link}'>{$bUrl['filename']}</a>";
					$embed .= "</object>";
					$embed .= "</object>";
				} else {				
					$embed = "<object classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' width='{$size['width']}' height='{$size['height']}'>";
					$embed .= "<param name='src' value='{$link}'>";
					$embed .= "<param name='controller' value='false'>";
					$embed .= "<param name='autoplay' value='true'>";
					$embed .= "</object>";
				}
				*/
				$embed = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embed .= "<param name='custommode' value='none' />";
				$embed .= "<param name='autoPlay' value='{$autoplay}' />";
				$embed .= "<param name='src' value='{$link}' />";
				$embed .= "<embed type='video/divx' src='{$link}' custommode='none' width='{$size['width']}' height='{$size['height']}'  autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embed .= "</embed>";
				$embed .= "</object>";
				break;
			case '11':	// MOV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<object width='{$size['width']}' height='{$size['height']}' classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
				$embed .= "<param name='src' value='{$link}'>";
				$embed .= "<param name='autoplay' value='{$autoplay}'>";
				$embed .= "<param name='controller' value='true'>";
				$embed .= "<embed src='{$link}' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' pluginspage='http://www.apple.com/quicktime/download/'></embed>";
				$embed .= "</object>";
				break;
			case '12':	// SWF Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<object classid='clsid:d27cdb6e-ae6d-11cf-96b8-444553540000' codebase='http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0' width='{$size['width']}' height='{$size['height']}' align='middle'>";
				$embed .= "<param name='allowScriptAccess' value='sameDomain' />";
				$embed .= "<param name='allowFullScreen' value='true' />";
				$embed .= "<param name='movie' value='{$link}' />";
				$embed .= "<param name='quality' value='high' />";
				$embed .= "<param name='wmode' value='transparent' />";
				$embed .= "<param name='bgcolor' value='#ffffff' />";
				$embed .= "<param name='autoplay' value='{$autoplay}'>";	
				$embed .= "<embed src='{$link}' quality='high' wmode='transparent' bgcolor='#ffffff' width='{$size['width']}' height='{$size['height']}' align='middle' allowScriptAccess='sameDomain' allowFullScreen='false' type='application/x-shockwave-flash' pluginspage='http://www.macromedia.com/go/getflashplayer' autoplay='{$autoplay}' />";
				$embed .= "</object>";
				break;
			case '13':	// FLV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					jQuery(document).ready(function() { 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 						
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					});
					}); 
				</script>";
				break;
			case '14':	// F4V Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					jQuery(document).ready(function() { 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: '/component/com_xmovie/assets/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					}); 
					});
				</script>";
				break;
			case '15':	// WMV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;				
				$embed = "<OBJECT id='mediaPlayer' width='{$size['width']}' height='{$size['height']}' classid='CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95' codebase='http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701' standby='Loading Microsoft Windows Media Player components...' type='application/x-oleobject'>";
      			$embed .= "<param name='fileName' value='{$link}'>";
	      		$embed .= "<param name='animationatStart' value='true'>";
	      		$embed .= "<param name='transparentatStart' value='true'>";
	      		$embed .= "<param name='autoStart' value='{$autoplay}'>";
	      		$embed .= "<param name='showControls' value='true'>";
	      		$embed .= "<param name='loop' value='false'>";
      			$embed .= "<EMBED type='application/x-mplayer2' pluginspage='http://microsoft.com/windows/mediaplayer/en/download/' id='mediaPlayer' name='mediaPlayer' displaysize='4' autosize='-1' bgcolor='darkblue' showcontrols='true' showtracker='-1' showdisplay='0' showstatusbar='-1' videoborder3d='-1' width='{$size['width']}' height='{$size['height']}' src='{$link}' autostart='{$autoplay}' designtimesp='5311' loop='false'>";
      			$embed .= "</EMBED>";
      			$embed .= "</OBJECT>";
				break;
			case '16':	// MP4 Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;					
				$embed = "<OBJECT classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' width='{$size['width']}' height='{$size['height']}' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
        		$embed .= "<param name='src' value='{$link}'>";
        		$embed .= "<param name='autoplay' value='{$autoplay}'>";
        		$embed .= "<param name='controller' value='true'>";
        		$embed .= "<param name='loop' value='false'>";
        		$embed .= "<EMBED src='{$link}' width={$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' loop='false' pluginspage='http://www.apple.com/quicktime/download/'>";
        		$embed .= "</EMBED>";
        		$embed .= "</OBJECT>";				
				break;
				/*
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					jQuery(document).ready(function() { 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: '/component/com_xmovie/assets/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					}); 
					});
				</script>";
				break;
				*/
			case '17':	// RM Local				
			case '18':	// RAM Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;								
				$embed = "<OBJECT id='rvocx' classid='clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA' width='{$size['width']}' height='{$size['height']}'>";
        		$embed .= "<param name='src' value='{$link}'>";
        		$embed .= "<param name='autostart' value='{$autoplay}'>";
        		$embed .= "<param name='controls' value='imagewindow'>";
        		$embed .= "<param name='console' value='video'>";
        		$embed .= "<param name='loop' value='false'>";
        		$embed .= "<EMBED src='{$link}' width='{$size['width']}' height='{$size['height']}' loop='false' type='audio/x-pn-realaudio-plugin' controls='imagewindow' console='video'  autostart='{$autoplay}'>";
        		$embed .= "</EMBED>";
        		$embed .= "</OBJECT>";				
				break;
			case '19':	// DIVX External
				$embed = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embed .= "<param name='custommode' value='none' />";
				$embed .= "<param name='autoPlay' value='{$autoplay}' />";
				$embed .= "<param name='src' value='{$movie->link}' />";
				$embed .= "<embed type='video/divx' src='{$movie->link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embed .= "</embed>";
				$embed .= "</object>";
				break;
			case '20':	// AVI External
				/*
				$bType = MovieHelper::getBrowserType();				
				if($bType['browser'] == 'Firefox' or $bType['browser'] == 'Safari') {
					$bUrl = pathinfo($movie->link);
					$embed = "<object type='video/quicktime' data='{$movie->link}' width='{$size['width']}' height='{$size['height']}'>";
					$embed .= "<param name='controller' value='false' >";
					$embed .= "<param name='autoplay' value='true'>";
					$embed .= "alt : <a href='{$movie->link}'>{$bUrl['filename']}</a>";
					$embed .= "</object>";
					$embed .= "</object>";
				} else {				
					$embed = "<object classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' width='{$size['width']}' height='{$size['height']}'>";
					$embed .= "<param name='src' value='{$movie->link}'>";
					$embed .= "<param name='controller' value='false'>";
					$embed .= "<param name='autoplay' value='true'>";
					$embed .= "</object>";
				}*/
				$embed = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embed .= "<param name='custommode' value='none' />";
				$embed .= "<param name='autoPlay' value='{$autoplay}' />";
				$embed .= "<param name='src' value='{$movie->link}' />";
				$embed .= "<embed type='video/divx' src='{$movie->link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embed .= "</embed>";
				$embed .= "</object>";
				break;
			case '21':	// MOV External
				$embed = "<object width='{$size['width']}' height='{$size['height']}' classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
				$embed .= "<param name='src' value='{$movie->link}'>";
				$embed .= "<param name='autoplay' value='{$autoplay}'>";
				$embed .= "<param name='controller' value='true'>";
				$embed .= "<embed src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' pluginspage='http://www.apple.com/quicktime/download/'></embed>";
				$embed .= "</object>";
				break;
			case '22':	// SWF External
				$embed = "<object classid='clsid:d27cdb6e-ae6d-11cf-96b8-444553540000' codebase='http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0' width='{$size['width']}' height='{$size['height']}' align='middle'>";
				$embed .= "<param name='allowScriptAccess' value='sameDomain' />";
				$embed .= "<param name='allowFullScreen' value='true' />";
				$embed .= "<param name='movie' value='{$movie->link}' />";
				$embed .= "<param name='quality' value='high' />";
				$embed .= "<param name='autoplay' value='{$autoplay}'>";
				$embed .= "<param name='wmode' value='transparent' />";
				$embed .= "<param name='bgcolor' value='#ffffff' />";
				$flashvars = '';
				if($movie->flashvars != '') {
					$embed .= "<param name='flashVars' value='{$movie->flashvars}' />";
					$flashvars = "flashvars='{$movie->flashvars}' ";
				}	
				$embed .= "<embed src='{$movie->link}' quality='high' wmode='transparent' bgcolor='#ffffff' {$flashvars} width='{$size['width']}' height='{$size['height']}' align='middle' allowScriptAccess='sameDomain' allowFullScreen='false' type='application/x-shockwave-flash' pluginspage='http://www.macromedia.com/go/getflashplayer' autoplay='{$autoplay}' />";
				$embed .= "</object>";
				break;
			case '23':	// FLV External
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						}
					}); 
				</script>";
				break;
			case '24':	// F4V External
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						}
					}); 
				</script>";
				break;
			case '25':	// WMV External
				$embed = "<OBJECT id='mediaPlayer' width='{$size['width']}' height='{$size['height']}' classid='CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95' codebase='http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701' standby='Loading Microsoft Windows Media Player components...' type='application/x-oleobject'>";
      			$embed .= "<param name='fileName' value='{$movie->link}'>";
	      		$embed .= "<param name='animationatStart' value='true'>";
	      		$embed .= "<param name='transparentatStart' value='true'>";
	      		$embed .= "<param name='autoStart' value='{$autoplay}'>";
	      		$embed .= "<param name='showControls' value='true'>";
	      		$embed .= "<param name='loop' value='false'>";
      			$embed .= "<EMBED type='application/x-mplayer2' pluginspage='http://microsoft.com/windows/mediaplayer/en/download/' id='mediaPlayer' name='mediaPlayer' displaysize='4' autosize='-1' bgcolor='darkblue' showcontrols='true' showtracker='-1' showdisplay='0' showstatusbar='-1' videoborder3d='-1' width='{$size['width']}' height='{$size['height']}' src='{$movie->link}' autoStart='{$autoplay}' designtimesp='5311' loop='false'>";
      			$embed .= "</EMBED>";
      			$embed .= "</OBJECT>";
				break;
			case '26':	// MP4 External
				$embed = "<object classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab' width='{$size['width']}' height='{$size['height']}' >";
				$embed .= "<param name='src' value='{$movie->link}' >";
				$embed .= "<param name='autoplay' value='{$autoplay}' >";
				$embed .= "<embed src='{$movie->link}' type='image/x-macpaint' pluginspage='http://www.apple.com/quicktime/download' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}'></embed>";
				$embed .= "</object>";
				break;
				/*
				$embed = "<div id='player_flv' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$embed .= "<script type='text/javascript'> 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						}
					}); 
				</script>";
				break;*/
			case '27':	// RM External
			case '28':	// RAM External
				$embed = "<OBJECT id='rvocx' classid='clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA' width='{$size['width']}' height='{$size['height']}'>";
        		$embed .= "<param name='src' value='{$movie->link}'>";
        		$embed .= "<param name='autostart' value='{$autoplay}'>";
        		$embed .= "<param name='controls' value='imagewindow'>";
        		$embed .= "<param name='console' value='video'>";
        		$embed .= "<param name='loop' value='false'>";
        		$embed .= "<EMBED src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' loop='false' type='audio/x-pn-realaudio-plugin' controls='imagewindow' console='video' autostart='{$autoplay}'>";
        		$embed .= "</EMBED>";
        		$embed .= "</OBJECT>";
				break;
			case '29':
				$embed = $movie->link;
			default:
				break;
		}
		
		return $embed;
	}
	
	function getShadowboxCode(&$movie, $num = '', $link = 0) {
		$component = JComponentHelper::getComponent( 'com_xmovie' );
  		$cfgParams = new JParameter( $component->params );

  		if($cfgParams->get('enable_movie_background', 0)) {
  			$urlParse = parse_url($movie->thumb);
			
			if(!array_key_exists('host', $urlParse)) {
  				$thumbPath = JURI::root().$cfgParams->get('movie_thumb_path', 'images/stories') . DS .$movie->thumb;
			} else {
				$thumbPath = $movie->thumb;
			}
  		} else {
  			$thumbPath = '';
  		}
  		
  		if($link) {
  			$relLink = '[xmoviesb]';
  		} else {
  			$relLink = '';
  		}
		$autoplay = $cfgParams->get('cview_autoplay', 0);
		$autoplaystring = "&autoplay={$autoplay}";
		$embed = array();
		$size = MovieHelper::calDimenions($movie->width, $movie->height, $cfgParams->get('single_max_width', 1024), $cfgParams->get('single_max_height', 768));
		
		switch($movie->type) {
			case '1':	// You Tube					
				if($movie->flashvars != '') {
					$movie->link = $movie->link .'?'.$movie->flashvars.$autoplaystring;
				} else {
					$movie->link = $movie->link .'?'.$autoplaystring;
				}
				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';				
				break;
			case '2':	// Daily Motion
				// Cannot be embedded				
				break;
			case '3':	// Meta Cafe
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';			
				break;
			case '4':	// My Video				
				// This will not be an option							
				break;
			case '5':	// Yahoo Video
				if($movie->flashvars != '') {
					$movie->link = $movie->link .'?'.$movie->flashvars.$autoplaystring;
				} else {
					$movie->link = $movie->link .'?'.$autoplaystring;
				}
				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';						
				break;
			case '6':	// Veoh
				if($movie->flashvars != '') {
					$movie->link = $movie->link .'?'.$movie->flashvars.$autoplaystring;
				} else {
					$movie->link = $movie->link .'?'.$autoplaystring;
				}
				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';				
				break;
			case '7':	// PP2G TV				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';
				break;
			case '8':	// Watch Me TV
				// This will not be supported
				/*if($movie->flashvars != '') {
					$movie->link = $movie->link .'?'.$movie->flashvars;
				}
				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="shadowbox;width='.$movie->width.';height='.$movie->height.';player=swf">';
				$embed['close'] = '</a>';	*/	
				break;
			case '9':	// DIVX Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embedded = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embedded .= "<param name='custommode' value='none' />";
				$embedded .= "<param name='autoPlay' value='{$autoplay}' />";
				$embedded .= "<param name='src' value='{$link}' />";
				$embedded .= "<embed type='video/divx' src='{$link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embedded .= "</embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';				
				break;
			case '10':	// AVI Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;					
				$embedded = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embedded .= "<param name='custommode' value='none' />";
				$embedded .= "<param name='autoPlay' value='{$autoplay}' />";
				$embedded .= "<param name='src' value='{$link}' />";
				$embedded .= "<embed type='video/divx' src='{$link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embedded .= "</embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '11':	// MOV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embedded = "<object width='{$size['width']}' height='{$size['height']}' classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
				$embedded .= "<param name='src' value='{$link}'>";
				$embedded .= "<param name='autoplay' value='{$autoplay}'>";
				$embedded .= "<param name='controller' value='true'>";
				$embedded .= "<embed src='{$link}' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' pluginspage='http://www.apple.com/quicktime/download/'></embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				
				break;
			case '12':	// SWF Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;				
				$embed['open'] = '<a href="'.$link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.'">';
				$embed['close'] = '</a>';
				break;
			case '13':	// FLV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
								
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() { 
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 

    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					});
					});
				</script>";

				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				
				break;
			case '14':	// F4V Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() {  
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: '/component/com_xmovie/assets/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 						
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					});
					}); 
				</script>";
				
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				
				break;
			case '15':	// WMV Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;				
				$embedded = "<OBJECT id='mediaPlayer' width='{$size['width']}' height='{$size['height']}' classid='CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95' codebase='http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701' standby='Loading Microsoft Windows Media Player components...' type='application/x-oleobject'>";
      			$embedded .= "<param name='fileName' value='{$link}'>";
	      		$embedded .= "<param name='animationatStart' value='true'>";
	      		$embedded .= "<param name='transparentatStart' value='true'>";
	      		$embedded .= "<param name='autoStart' value='{$autoplay}'>";
	      		$embedded .= "<param name='showControls' value='true'>";
	      		$embedded .= "<param name='loop' value='false'>";
      			$embedded .= "<EMBED type='application/x-mplayer2' pluginspage='http://microsoft.com/windows/mediaplayer/en/download/' id='mediaPlayer' name='mediaPlayer' displaysize='4' autosize='-1' bgcolor='darkblue' showcontrols='true' showtracker='-1' showdisplay='0' showstatusbar='-1' videoborder3d='-1' width='{$size['width']}' height='{$size['height']}' src='{$link}' autostart='{$autoplay}' designtimesp='5311' loop='false'>";
      			$embedded .= "</EMBED>";
      			$embedded .= "</OBJECT>";
      			$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '16':	// MP4 Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;					
				$embedded = "<OBJECT classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' width='{$size['width']}' height='{$size['height']}' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
        		$embedded .= "<param name='src' value='{$link}'>";
        		$embedded .= "<param name='autoplay' value='{$autoplay}'>";
        		$embedded .= "<param name='controller' value='true'>";
        		$embedded .= "<param name='loop' value='false'>";
        		$embedded .= "<EMBED src='{$link}' width={$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' loop='false' pluginspage='http://www.apple.com/quicktime/download/'>";
        		$embedded .= "</EMBED>";
        		$embedded .= "</OBJECT>";
        		$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
        		
        		$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';				
				break;
				/*
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() {  
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: '/component/com_xmovie/assets/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 						
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					});
					}); 
				</script>";
				
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				
				break;*/
			case '17':	// RM Local				
			case '18':	// RAM Local
				$link = COM_XMOVIE_BASEURL . DS . $movie->link;								
				$embedded = "<OBJECT id='rvocx' classid='clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA' width='{$size['width']}' height='{$size['height']}'>";
        		$embedded .= "<param name='src' value='{$link}'>";
        		$embedded .= "<param name='autostart' value='{$autoplay}'>";
        		$embedded .= "<param name='controls' value='imagewindow'>";
        		$embedded .= "<param name='console' value='video'>";
        		$embedded .= "<param name='loop' value='false'>";
        		$embedded .= "<EMBED src='{$link}' width='{$size['width']}' height='{$size['height']}' loop='false' type='audio/x-pn-realaudio-plugin' controls='imagewindow' console='video' autostart='{$autoplay}'>";
        		$embedded .= "</EMBED>";
        		$embedded .= "</OBJECT>";
        		$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
        		
        		$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';			
				break;
			case '19':	// DIVX External
				$embedded = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embedded .= "<param name='custommode' value='none' />";
				$embedded .= "<param name='autoPlay' value='{$autoplay}' />";
				$embedded .= "<param name='src' value='{$movie->link}' />";
				$embedded .= "<embed type='video/divx' src='{$movie->link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embedded .= "</embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '20':	// AVI External				
				$embedded = "<object classid='clsid:67DABFBF-D0AB-41fa-9C46-CC0F21721616' width='{$size['width']}' height='{$size['height']}' codebase='http://go.divx.com/plugin/DivXBrowserPlugin.cab'>";
				$embedded .= "<param name='custommode' value='none' />";
				$embedded .= "<param name='autoPlay' value='{$autoplay}' />";
				$embedded .= "<param name='src' value='{$movie->link}' />";
				$embedded .= "<embed type='video/divx' src='{$movie->link}' custommode='none' width='{$size['width']}' height='{$size['height']}' autoPlay='{$autoplay}' pluginspage='http://go.divx.com/plugin/download/'>";
				$embedded .= "</embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '21':	// MOV External
				$embedded = "<object width='{$size['width']}' height='{$size['height']}' classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab'>";
				$embedded .= "<param name='src' value='{$movie->link}'>";
				$embedded .= "<param name='autoplay' value='{$autoplay}'>";
				$embedded .= "<param name='controller' value='true'>";
				$embedded .= "<embed src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}' controller='true' pluginspage='http://www.apple.com/quicktime/download/'></embed>";
				$embed .= "</object>";
				$embedded = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '22':	// SWF External				
				$embed['open'] = '<a href="'.$movie->link.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$movie->width.';height='.$movie->height.'">';
				$embed['close'] = '</a>';
				break;
			case '23':	// FLV External
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() {
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					}); 
					});
				</script>";
				
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '24':	// F4V External
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() {
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					}); 
					});
				</script>";
				
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '25':	// WMV External
				$embedded = "<OBJECT id='mediaPlayer' width='{$size['width']}' height='{$size['height']}' classid='CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95' codebase='http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701' standby='Loading Microsoft Windows Media Player components...' type='application/x-oleobject'>";
      			$embedded .= "<param name='fileName' value='{$movie->link}'>";
	      		$embedded .= "<param name='animationatStart' value='true'>";
	      		$embedded .= "<param name='transparentatStart' value='true'>";
	      		$embedded .= "<param name='autoStart' value='{$autoplay}'>";
	      		$embedded .= "<param name='showControls' value='true'>";
	      		$embedded .= "<param name='loop' value='false'>";
      			$embedded .= "<EMBED type='application/x-mplayer2' pluginspage='http://microsoft.com/windows/mediaplayer/en/download/' id='mediaPlayer' name='mediaPlayer' displaysize='4' autosize='-1' bgcolor='darkblue' showcontrols='true' showtracker='-1' showdisplay='0' showstatusbar='-1' videoborder3d='-1' width='{$size['width']}' height='{$size['height']}' src='{$movie->link}' autostart='{$autoplay}' designtimesp='5311' loop='false'>";
      			$embedded .= "</EMBED>";
      			$embedded .= "</OBJECT>";
      			$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '26':	// MP4 External
				$embedded = "<object classid='clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B' codebase='http://www.apple.com/qtactivex/qtplugin.cab' width='{$size['width']}' height='{$size['height']}' >";
				$embedded .= "<param name='src' value='{$movie->link}' >";
				$embedded .= "<param name='autoplay' value='{$autoplay}' >";
				$embedded .= "<embed src='{$movie->link}' type='image/x-macpaint' pluginspage='http://www.apple.com/quicktime/download' width='{$size['width']}' height='{$size['height']}' autoplay='{$autoplay}'></embed>";
				$embedded .= "</object>";
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
        		
        		$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
				/*
				$embedded = "<div id='player_flv{$num}' style='width:{$size['width']}px; height:{$size['height']}px;'></div><div id='info'></div>";
				$script = "<script type='text/javascript'> 
					jQuery(document).ready(function() {
					// Flowplayer installation with Flashembed configuration 
					flowplayer('player_flv{$num}', { 
 
    					// our Flash component 
    					src: 'http://releases.flowplayer.org/swf/flowplayer-3.2.7.swf', 
 
    					// we need at least this Flash version 
    					version: [9, 115], 
 
    					// older versions will see a custom message 
    					onFail: function()  { 
        				document.getElementById('info').innerHTML = 
            			'You need the latest Flash version to see MP4 movies. ' + 
            			'Your version is ' + this.getVersion(); 
    				} 
 
    				// here is our third argument which is the Flowplayer configuration 
					}, { 
    					clip: {
    						url : '{$movie->link}',
    						autoPlay: {$autoplay},
    						wmode: 'transparent'
    					},
    					canvas: {
							backgroundImage: 'url(".$thumbPath.")'
						} 
					}); 
					});
				</script>";
				
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
				
				$embed['open'] = $script.$divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;*/
			case '27':	// RM External
			case '28':	// RAM External
				$embedded = "<OBJECT id='rvocx' classid='clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA' width='{$size['width']}' height='{$size['height']}'>";
        		$embedded .= "<param name='src' value='{$movie->link}'>";
        		$embedded .= "<param name='autostart' value='{$autoplay}'>";
        		$embedded .= "<param name='controls' value='imagewindow'>";
        		$embedded .= "<param name='console' value='video'>";
        		$embedded .= "<param name='loop' value='false'>";
        		$embedded .= "<EMBED src='{$movie->link}' width='{$size['width']}' height='{$size['height']}' loop='false' type='audio/x-pn-realaudio-plugin' controls='imagewindow' console='video' autostart='{$autoplay}'>";
        		$embedded .= "</EMBED>";
        		$embedded .= "</OBJECT>";
        		$divcont = "<div id='video-cont{$num}' style='display:none;'>{$embedded}</div>";
        		
        		$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			case '29':
				$size['width'] = $size['width'] + $cfgParams->get('html_offset_width', 25);
				$size['height'] = $size['height'] + $cfgParams->get('html_offset_height', 25);
				$divcont = "<div id='video-cont{$num}' style='display:none;'>{$movie->link}</div>";
				$embed['open'] = $divcont.'<a href="#video-cont'.$num.'" title="'.htmlspecialchars($movie->name).'" rel="lightbox'.$relLink.';width='.$size['width'].';height='.$size['height'].'">';
				$embed['close'] = '</a>';
				break;
			default:
				break;
		}
		
		return $embed;
	}
	
	function getFlowPlayerScript(&$movie) {
		
	}
	
	function calDimenions($origWidth, $origHeight, $maxWidth, $maxHeight) {
    
    	$myDimenions = array();
    
    	if (empty($origHeight) || $origHeight == "") {
        	$origHeight = $maxHeight;
    	}
        
    	if (empty($origWidth) || $origWidth == "") {
        	$origWidth = $maxWidth;
    	}
        
    	$imgratio = $origWidth / $origHeight;
    	
    	if($imgratio > 1) {
    		$newwidth = $maxWidth;
    		$newheight = $maxWidth / $imgratio;
    	} else {
    		$newwidth = $maxWidth;
    		$newheight = $maxWidth * $imgratio;
    	}
    	    
    	$myDimensions['width'] = round($newwidth);
    	$myDimensions['height'] = round($newheight);
    	
   		return $myDimensions;
	}
	
	function getXMovieItemId($cid = null) {
		$com = JComponentHelper::getComponent('com_xmovie');
		$db =& JFactory::getDBO();
		
		$query = "SELECT id FROM #__menu WHERE published = '1' AND componentid='{$com->id}' AND link LIKE '%main%' LIMIT 1";
		$db->setQuery($query);		
		$row = $db->loadObject();
			
		if(isset($row) && !empty($row)) {
			return $row;
		} else {
			if(!isset($row) || empty($row) && $cid != null) {
				$query = "SELECT id, link FROM #__menu WHERE published = '1' AND componentid='{$com->id}' AND link LIKE '%view=category%'";
				$db->setQuery($query);		
				$row = $db->loadObjectList();	
				
				if(isset($row) || !empty($row)) {					
					foreach($row as $cRow) {
						$parentId = '';
						$url = JURI::getInstance($cRow->link);
						$parentId = MovieHelper::isParentId($url->getVar('id'), $cid, $db);
						
						if($parentId){
							$row = new stdClass;
							$row->id = $cRow->id;
							return $row;
						}
					}
					
					$row = '';
					$row = new stdClass;
					$row->id = '';
					return $row;
				} else {
					$row = '';					
					$row = new stdClass;
					$row->id = '';
					return $row;
				}
			} else {
				$row = '';
				$row = new stdClass;
				$row->id = '';
				return $row;
			}
		}		
		
		return $row;		
	}
	
	function isParentId($id, $cid, $db) {	
		if($id == $cid) {
			return true;
		}
		
		if($cid != '') {
			$query = "SELECT pid FROM #__xmovie_categories WHERE id='{$cid}' LIMIT 1";
			$db->setQuery($query);
			$row = $db->loadObject();
			
			if($row->pid == $id) {
				return true;
			} else if($row->pid == 1 || $row->pid == 0) {
				return false;
			} else {
				$result = MovieHelper::isParentId($id, $row->pid, $db);
				
				if($result) {
					return true;
				} else {
					return false;
				}
			}
		}
		
		return false;
	}
	
	function getComponentId() {		
		$com = JComponentHelper::getComponent('com_xmovie');
		$query = "SELECT id FROM #__menu WHERE published = '1' AND componentid='{$com->id}' AND link LIKE '%main%' LIMIT 1";
		$db =& JFactory::getDBO();
		$db->setQuery($query);		
		$row = $db->loadObject();
		
		if(!isset($row) || empty($row)) {
			$row = new stdClass;
			$row->id = null;
		}
		
		$currentId = JRequest::getVar('Itemid', '');

		if(isset($row->id) && $currentId != $row->id) {
			$row->id = $currentId;
		} else if($row->id == '') {
			$row->id = $currentId;
		}
		
		return $row;
	}
	
	function getMovieId($id) {
		jimport('joomla.environment.uri');
		$com = JComponentHelper::getComponent('com_xmovie');
		$query = "SELECT id, link FROM #__menu WHERE componentid='{$com->id}' AND link LIKE '%main%'";
		$db =& JFactory::getDBO();
		$db->setQuery($query);		
		$links = $db->loadObjectList();
		
		$main = new stdClass;
		$views = array();
		$itemid = '';
		
		if(count($links) > 0) {
			foreach($links as $link) {
				$currURI = JURI::getInstance($link->link);

				if($currURI->getVar('view') == 'category') {
					$itemid = $currURI->getVar('id');
					$views[$itemid] = new stdClass;
					$views[$itemid]->id = $link->id;
				} else if ($currURI->getVar('view') == 'main') {
					$main->id = $link->id;
				} else if ($currURI->getVar('view') != 'single') {
					$main->id = $link->id;
				}
			}
		}
		
		if(array_key_exists($id, $views)) {
			return $views[$id];
		} else {
			return $main;
		}
	}
	
	function setCookieParams() {
		global $mainframe;

		$component = JComponentHelper::getComponent( 'com_xmovie' );
  		$cfgParams = new JParameter( $component->params );
  		
  		$expire = time()+60*60;
  		$path = '/';
  		
		if(!defined('COM_MEDIA_BASE')) {
  			$imagPath = $cfgParams->get('movie_thumb_path', 'images/stories');
  			$baseDir = JPATH_ROOT.DS.$imagPath;
  		} else {
  			$baseDir = COM_XMOVIE_BASE;
  		}
  		
  		$pObject = (object) array();
  		
  		$pObject->admin_img_width = $cfgParams->get('rWidth_admin', 75);
  		$pObject->admin_img_height = $cfgParams->get('rHeight_admin', 75);
  		$pObject->cat_img_width = $cfgParams->get('cview_cat_width', 75);
  		$pObject->cat_img_height = $cfgParams->get('cview_cat_height', 75);
  		$pObject->mov_img_width = $cfgParams->get('cview_mov_width', 75);
  		$pObject->mov_img_height = $cfgParams->get('cview_mov_height', 75);
  		$pObject->base_dir = $baseDir;

  		$fileParams = serialize($pObject);
  		$fileParams = gzcompress($fileParams);
  		$fileParams = base64_encode($fileParams);
  		
  		setCookie('xmovie_cookie', $fileParams, $expire, $path);
	}
	
	function getCategory($id) {
		$query = "SELECT cid FROM #__xmovie_movies WHERE published = '1' AND id='{$id}' LIMIT 1";
		$db =& JFactory::getDBO();
		$db->setQuery($query);
		$row = $db->loadObject();
		return $row;
	}
		
	function getCategorySlug() {
		$component = JComponentHelper::getComponent( 'com_xmovie' );
  		$cfgParams = new JParameter( $component->params );
		$alias = $cfgParams->get('cat_alias_name', 'category');
		
		return $alias;
	}
	
	function getMovieSlug() {
		$component = JComponentHelper::getComponent( 'com_xmovie' );
  		$cfgParams = new JParameter( $component->params );
		$alias = $cfgParams->get('mov_alias_name', 'movie');
		
		return $alias;
	}
		
	function getCategoryTree($data, $tree, $id = 0, $currentId) {		
		
		// Ordering
		$countItemsInCat = 0;
		foreach ($data as $key) {
			static $iCT = 0;// All displayed items
	
			if ($key->pid == $id && $currentId != $id && $currentId != $key->id ) {	
							
				$tree[$iCT] 					= new JObject();				
				
				// Ordering MUST be solved here
				if ($countItemsInCat > 0) {
					$tree[$iCT]->orderup				= 1;
				} else {
					$tree[$iCT]->orderup 				= 0;
				}
				
				if ($countItemsInCat < ($key->countid - 1)) {
					$tree[$iCT]->orderdown 				= 1;
				} else {
					$tree[$iCT]->orderdown 				= 0;
				}
								
				$tree[$iCT]->id 				= $key->id;
				$tree[$iCT]->pid				= $key->pid;
				$tree[$iCT]->name 				= $key->name;
				$tree[$iCT]->thumb				= $key->thumb;
				$tree[$iCT]->hits				= $key->hits;
				$tree[$iCT]->banner				= $key->banner;
				$tree[$iCT]->quicktake			= $key->quicktake;
				$tree[$iCT]->description		= $key->description;
				$tree[$iCT]->access				= $key->access;
				$tree[$iCT]->groupname			= $key->groupname;
				$tree[$iCT]->creation_date		= $key->creation_date;
				$tree[$iCT]->published			= $key->published;
				$tree[$iCT]->ordering			= $key->ordering;
				$tree[$iCT]->metakey			= $key->metakey;
				$tree[$iCT]->metadesc			= $key->metadesc;
				$tree[$iCT]->metaauthor			= $key->metaauthor;
				$tree[$iCT]->metarobots			= $key->metarobots;
				
				$iCT++;
				
				$tree = MovieHelper::getCategoryTree($data, $tree, $key->id, $currentId );
				$countItemsInCat++;
			} 
		} 
		
		return($tree);
	}
}